;--------------------------------
;NSIS OpenLibraries Installation Script
;--------------------------------

!ifndef VERSION
  !define VERSION 0_5_0
  !define VERSION_DOT 0.5.0
!endif

;--------------------------------
;Configuration

!ifdef OUTFILE
  OutFile "${OUTFILE}"
!else
  OutFile openlibraries-${VERSION_DOT}-runtime.exe
!endif

SetCompressor /SOLID lzma

InstType "Full"

InstallDir $PROGRAMFILES\OpenLibraries
InstallDirRegKey HKLM Software\OpenLibraries ""

;--------------------------------
;Header Files

!define ALL_USERS

!include "MUI.nsh"
!include "Sections.nsh"
!include "add_to_path.nsh"
!include "write_env_str.nsh"

;Names
Name "OpenLibraries"
Caption "OpenLibraries (v.${VERSION_DOT}) Setup"

;Interface Settings
!define MUI_ABORTWARNING

!define MUI_HEADERIMAGE
!define MUI_HEADERIMAGE_BITMAP "openlibraries.bmp" ; optional
!define MUI_WELCOMEFINISHPAGE_BITMAP "${NSISDIR}\Contrib\Graphics\Wizard\orange.bmp"

!define MUI_COMPONENTSPAGE_SMALLDESC

;Pages
!define MUI_WELCOMEPAGE_TITLE "Welcome to the OpenLibraries (v.${VERSION_DOT}) Setup Wizard"
!define MUI_WELCOMEPAGE_TEXT "This wizard will guide you through the installation of OpenLibraries.\r\n\r\n$_CLICK"

!insertmacro MUI_PAGE_WELCOME
!insertmacro MUI_PAGE_LICENSE "..\COPYING"
;!insertmacro MUI_PAGE_COMPONENTS
;!insertmacro MUI_PAGE_DIRECTORY
!insertmacro MUI_PAGE_INSTFILES

!define MUI_FINISHPAGE_LINK "Visit the web site for the latest news, FAQs and support"
!define MUI_FINISHPAGE_LINK_LOCATION "http://www.openlibraries.org/"

!insertmacro MUI_PAGE_FINISH

!insertmacro MUI_UNPAGE_CONFIRM
!insertmacro MUI_UNPAGE_INSTFILES

;--------------------------------
;Languages

!insertmacro MUI_LANGUAGE "English"

 Function .onInit
 
  
  ReadRegStr $R0 HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\OpenLibraries" "UninstallString"
  StrCmp $R0 "" 0 +2
  Return

  ;IfSilent 0 +4
  ;ReadRegStr $R1 HKLM "Software\OpenLibraries" "Version"
  ;StrCmp $R1 "${VERSION}" 0 +2
  ;Abort

  ;MessageBox MB_OKCANCEL|MB_ICONEXCLAMATION
  ;"OpenLibraries v${VERSION} is already installed. $\n$\nClick `OK` to remove the
  ;previous version or `Cancel` to cancel this upgrade." 
  ;/SD IDOK IDOK uninst
  ;Abort
  
  ;Run the uninstaller
  ;uninst:
  ClearErrors
	  
  ;Start the uninstaller using the option to not copy itself
  ExecWait '$R0 /S _?=$INSTDIR'
	 
  IfErrors no_remove_uninstaller
  ; In most cases the uninstall is successful at this point.
  ; You may also consider using a registry key to check whether 
  ; the user has chosen to uninstall. If you are using an uninstaller
  ; components page, make sure all sections are uninstalled.
  Return
	    
  no_remove_uninstaller:
  MessageBox MB_ICONEXCLAMATION \
  "Unable to remove previous version of OpenLibraries"
  Abort
 
FunctionEnd

;Section "MSXML4 SP2" msxml2

;  SectionIn 1
;  SetOutPath $INSTDIR

;  SetOverwrite on
;  SetShellVarContext all

;  SetDetailsPrint textonly
;  DetailPrint "Installing MSXML4 SP2..."
;  SetDetailsPrint listonly

;  SetOutPath $INSTDIR
;  File "msxml4.msi"

;  ExecWait 'MsiExec.exe /qn /x{2AEBE10C-D819-4EBF-BC60-03BF2327D340}'	; uninstall MSXML4 if it's there
;  ExecWait 'MsiExec.exe /qn /x{35343FF7-939B-401A-87B3-FF90A5123D88}'	; uninstall MSXML4 SP1 if it's there
;  ExecWait 'MsiExec.exe /qn /i "$INSTDIR\msxml4.msi"'
  
;  WriteRegStr HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\OpenLibraries" "msxml2" ""

;  SetDetailsPrint both

;SectionEnd
 
Section "OpenLibraries" SecCore

  SetDetailsPrint textonly
  DetailPrint "Installing OpenLibraries..."
  SetDetailsPrint listonly

  SectionIn 1 2 3 RO
  SetOutPath $INSTDIR

  SetOverwrite on

  SetOutPath $INSTDIR\bin
  File "..\src\openpluginlib\pl\Multi-threaded Release DLL\openpluginlib_pl-vc80-r-${VERSION}.dll"
  File "..\src\openimagelib\il\Multi-threaded Release DLL\openimagelib_il-vc80-r-${VERSION}.dll"
  File "..\src\openmedialib\ml\Multi-threaded Release DLL\openmedialib_ml-vc80-r-${VERSION}.dll"
  File "..\src\openobjectlib\sg\Multi-threaded Release DLL\openobjectlib_sg-vc80-r-${VERSION}.dll"
  File "..\src\openassetlib\al\Multi-threaded Release DLL\openassetlib_al-vc80-r-${VERSION}.dll"

  ; dependencies
  File "C:\Program Files\FFmpeg\lib\avcodec-51.dll"
  File "C:\Program Files\FFmpeg\lib\avformat-52.dll"
  File "C:\Program Files\FFmpeg\lib\avutil-49.dll"
  File "C:\Program Files\FFmpeg\lib\libmp3lame-0.dll"
;  File "C:\Program Files\FFmpeg\lib\libogg-0.dll"
  File "C:\OpenEXR\bin\Release\Half.dll"
  File "C:\OpenEXR\bin\Release\Iex.dll"
  File "C:\OpenEXR\bin\Release\IlmImf.dll"
  File "C:\OpenEXR\bin\Release\Half.dll"
  File "C:\OpenEXR\bin\Release\IlmThread.dll"
  File "C:\OpenEXR\bin\Release\Imath.dll"
;  File "C:\SDL-1.2.11\lib\SDL.dll"
  File "C:\Program Files\NVIDIA Corporation\Cg\bin\cg.dll"
  File "C:\Program Files\NVIDIA Corporation\Cg\bin\cgGL.dll"
  File "C:\Windows\system32\glew32.dll"
  File "c:\Windows\system32\python25.dll"
  File "c:\boost\lib\boost_python-vc80-mt-1_34_1.dll"
  File "c:\boost\lib\boost_thread-vc80-mt-1_34_1.dll"
  File "c:\boost\lib\boost_regex-vc80-mt-1_34_1.dll"
  File "c:\boost\lib\boost_iostreams-vc80-mt-1_34_1.dll"
  File "c:\boost\lib\boost_filesystem-vc80-mt-1_34_1.dll"
  File "C:\zlib\lib\zlib1.dll"

  ; Python
  SetOutPath $INSTDIR\python\release
  File "..\src\openpluginlib\py\Multi-threaded Release DLL\openpluginlib.pyd"
  File "..\src\openimagelib\py\Multi-threaded Release DLL\openimagelib.pyd"
  File "..\src\openmedialib\py\Multi-threaded Release DLL\openmedialib.pyd"
  File "..\src\openassetlib\py\Multi-threaded Release DLL\openassetlib.pyd"
  
  SetOutPath $INSTDIR\plugins\release
  File "..\src\openimagelib\plugins\dds\dds_plugin.opl"
  File "..\src\openimagelib\plugins\dds\Multi-threaded Release DLL\openimagelib_dds-vc80-r-${VERSION}.dll"
  File "..\src\openimagelib\plugins\dpx\dpx_plugin.opl"
  File "..\src\openimagelib\plugins\dpx\Multi-threaded Release DLL\openimagelib_dpx-vc80-r-${VERSION}.dll"
  File "..\src\openimagelib\plugins\exr\exr_plugin.opl"
  File "..\src\openimagelib\plugins\exr\Multi-threaded Release DLL\openimagelib_exr-vc80-r-${VERSION}.dll"
  File "..\src\openimagelib\plugins\gdi+\gdi_plugin.opl"
  File "..\src\openimagelib\plugins\gdi+\Multi-threaded Release DLL\openimagelib_gdi-vc80-r-${VERSION}.dll"
  File "..\src\openimagelib\plugins\hdr\hdr_plugin.opl"
  File "..\src\openimagelib\plugins\hdr\Multi-threaded Release DLL\openimagelib_hdr-vc80-r-${VERSION}.dll"
  File "..\src\openimagelib\plugins\sgi\sgi_plugin.opl"
  File "..\src\openimagelib\plugins\sgi\Multi-threaded Release DLL\openimagelib_sgi-vc80-r-${VERSION}.dll"
  File "..\src\openimagelib\plugins\tga\tga_plugin.opl"
  File "..\src\openimagelib\plugins\tga\Multi-threaded Release DLL\openimagelib_tga-vc80-r-${VERSION}.dll"
  File "..\src\openmedialib\plugins\avformat\avformat_plugin.opl"
  File "..\src\openmedialib\plugins\avformat\Multi-threaded Release DLL\openmedialib_avformat-vc80-r-${VERSION}.dll"
  File "..\src\openmedialib\plugins\oil\oil_plugin.opl"
  File "..\src\openmedialib\plugins\oil\Multi-threaded Release DLL\openmedialib_oil-vc80-r-${VERSION}.dll"
  File "..\src\openmedialib\plugins\openal\openal_plugin.opl"
  File "..\src\openmedialib\plugins\openal\Multi-threaded Release DLL\openmedialib_openal-vc80-r-${VERSION}.dll"
  File "..\src\openmedialib\plugins\gensys\gensys_plugin.opl"
  File "..\src\openmedialib\plugins\gensys\Multi-threaded Release DLL\openmedialib_gensys-vc80-r-${VERSION}.dll"
  File "..\src\openmedialib\plugins\quicktime\quicktime_plugin.opl"
  File "..\src\openmedialib\plugins\quicktime\Multi-threaded Release DLL\openmedialib_quicktime-vc80-r-${VERSION}.dll"
; File "..\src\openmedialib\plugins\sdl\sdl_plugin.opl"
; File "..\src\openmedialib\plugins\sdl\Multi-threaded Release DLL\openmedialib_sdl-vc80-r-${VERSION}.dll"
;  File "..\src\openobjectlib\plugins\obj\obj_plugin.opl"
;  File "..\src\openobjectlib\plugins\obj\Multi-threaded Release DLL\openobjectlib_obj-vc80-r-${VERSION}.dll"
;  File "..\src\openobjectlib\plugins\X3D\x3d_plugin.opl"
;  File "..\src\openobjectlib\plugins\X3D\Multi-threaded Release DLL\openobjectlib_x3d-vc80-r-${VERSION}.dll"
;  File "..\src\openobjectlib\plugins\Collada\collada_plugin.opl"
;  File "..\src\openobjectlib\plugins\Collada\Multi-threaded Release DLL\openobjectlib_collada-vc80-r-${VERSION}.dll"
;  File "..\src\openeffectslib\plugins\tonemap\tonemap_plugin.opl"
;  File "..\src\openeffectslib\plugins\tonemap\Multi-threaded Release DLL\openeffectslib_tonemap-vc80-r-${VERSION}.dll"
  File "..\src\openassetlib\plugins\sqlite\sqlite3_metadata_plugin.opl"
  File "..\src\openassetlib\plugins\sqlite\Multi-threaded Release DLL\openassetlib_sqlite-vc80-r-${VERSION}.dll"
    
  SetOutPath $INSTDIR\shaders
 ; File "..\effects\shaders\sepia.cg"
 ; File "..\effects\shaders\sepia.frag"
 ; File "..\effects\shaders\fixed_function.vert"
  File "..\effects\shaders\passthrough.vert"
  File "..\effects\shaders\passthrough.cg"
  File "..\effects\shaders\rgb_to_yuv444.frag"
  File "..\effects\shaders\yuv444_to_rgb.frag"
  File "..\effects\shaders\yuv444_to_rgb.cg"
  File "..\effects\shaders\yuv444_to_rgb_deinterlace.cg"
  File "..\effects\shaders\yuv444_to_rgb_deinterlace.frag"
  File "..\effects\shaders\histogram.cg"
  File "..\effects\shaders\histogram.frag"
  File "..\effects\shaders\channel_selector.cg"
  File "..\effects\shaders\channel_selector.frag"
 
  SetDetailsPrint both

  ;--------------------------------
  ;Path environment variable

  Push "$INSTDIR\bin"
  Call AddToPath

  ;--------------------------------
  ;Python environment variable
  Push "PYTHONPATH"
  Push "$INSTDIR\python\Release"
  Call WriteEnvStr

  ;--------------------------------
  ;Registry

  WriteRegStr HKLM "Software\OpenLibraries" "CurrentVersion"        "${VERSION_DOT}"
  WriteRegStr HKLM "Software\OpenLibraries" "Installation type"     "Runtime"
  WriteRegStr HKLM "Software\OpenLibraries" "PluginsDirRelease"     "$INSTDIR\plugins\release"
  WriteRegStr HKLM "Software\OpenLibraries" "KernelsDir"            "$INSTDIR\kernels"
  WriteRegStr HKLM "Software\OpenLibraries" "ShadersDir"            "$INSTDIR\shaders"

  WriteRegStr HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\OpenLibraries" "DisplayName" "OpenLibraries"
  WriteRegStr HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\OpenLibraries" "UninstallString" "$INSTDIR\uninst-openlibraries.exe"

SectionEnd

Section -post

	WriteUninstaller $INSTDIR\uninst-openlibraries.exe

SectionEnd

;--------------------------------
;Uninstaller Section

Section Uninstall

  SetDetailsPrint textonly
  DetailPrint "Deleting Files..."
  SetDetailsPrint listonly

  DeleteRegKey HKLM "Software\OpenLibraries"
  DeleteRegKey HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\OpenLibraries"

  Push "$INSTDIR\bin"
  Call un.RemoveFromPath

  Push "PYTHONPATH"
  Call un.DeleteEnvStr

  RMDir /r $INSTDIR\bin
  RMDir /r $INSTDIR\include
  RMDir /r $INSTDIR\lib
  RMDir /r $INSTDIR\plugins
  RMDir /r $INSTDIR\shaders
  RMDir /r $INSTDIR

  SetDetailsPrint both
 
 SectionEnd

