#!/bin/sh
# Copyright (c) 2007 Editopia.
# From the script in HDRFlow - www.hdrflow.com
# Released under the GPLv3 or later.
# Unification of build and distribution:
#	recurses into lib and app directories, building each in turn.
#	updates Boost version in VC project files.
#   updates version.
#	builds OS/X frameworks

check_status( )
{
	if [ $1 = 0 ]
	then
		echo " succeeded."
	else
		echo " failed. Aborting..."
		exit 1
	fi
}

if [ $# -eq "0" ]
then
	echo "usage: `basename $0` options (-evbfioxupasmd)"
	echo "	-e ) update Boost version (Win32 only)"
	echo "	-v ) boost old version"
	echo "	-b ) boost new version"
	echo "	-f ) build OS/X frameworks"
	echo "	-x ) framework version"
	echo "	-o ) output_directory"
	echo "	-i ) install name prefix"
	echo "	-u ) build OS/X umbrella framework"
	echo "	-p ) build installer"
	echo "	-s ) strip symbols"
	echo "	-d ) update OpenLibraries version"
	echo
	echo "		Examples:"
	echo "			sh ./unity.sh -fu - build OS/X umbrella framework"
	exit 1
fi

PLATFORM=`uname -s`

update_boost_version=0
boost_old_version=1_34
boost_new_version=1_34_1

update_olibs_version=0
olibs_old_version=0_4_0
olibs_new_version=0_5_0

build_frameworks=0
build_umbrella=0
build_installer=0
build_application=0
build_maya_plugins=0
framework_version="A"
install_name_prefix="/Library/Frameworks"
strip_symbols=0

output_directory="."

openlibraries_configure_options="--with-boostprefix=/usr/local --with-boostversion=1_34_1 --with-pythonversion=2.3 --with-boostthreadruntime=mt --enable-universalbinaries --disable-dependency-tracking --with-glewprefix=/usr/local --with-tiffprefix=/usr/local"

while getopts "v:b:efi:o:x:upasmd" option
do
	case $option in
		e ) update_boost_version=1;;
		v ) boost_old_version="$OPTARG";;
		b ) boost_new_version="$OPTARG";;
		f ) build_frameworks=1;;
		x ) framework_version="$OPTARG";;
		o ) output_directory="$OPTARG";;
		i ) install_name_prefix="$OPTARG";;
		u ) build_umbrella=1;;
		p ) build_installer=1;;
		a ) build_application=1;;
		s ) strip_symbols=1;;
		m ) build_maya_plugins=1;;
		d ) update_olibs_version=1;;
	esac
done

shift $(($OPTIND - 1))

if [ "$update_boost_version" -eq 1 ]
then
	find . -type f -name '*.vcproj' -print | while read i
	do	cp $i $i.unity.bak &&
		sed -e "s|boost-$boost_old_version|boost-$boost_new_version|g" -e "s|$boost_old_version.lib|$boost_new_version.lib|g" $i.unity.bak > $i &&
		rm -f $i.unity.bak
	done
fi

if [ "$update_olibs_version" -eq 1 ]
then
	find . -type f -name '*.vcproj' -print | while read i
	do	cp $i $i.unity.bak &&
		sed -e "s|-$olibs_old_version.dll|-$olibs_new_version.dll|g" -e "s|$olibs_old_version.lib|$olibs_new_version.lib|g" $i.unity.bak > $i &&
		rm -f $i.unity.bak
	done

	find . -type f -name '*.opl' -print | while read i
	do	cp $i $i.unity.bak &&
		sed -e "s|-$olibs_old_version.dll|-$olibs_new_version.dll|g" $i.unity.bak > $i &&
		rm -f $i.unity.bak
	done
	
	find lib/openlibraries -type f -name 'config.hpp' -print | while read i
	do	cp $i $i.unity.bak &&
		sed -e "s|$olibs_old_version|$olibs_new_version|g" $i.unity.bak > $i &&
		rm -f $i.unity.bak
	done
fi

make_framework_dir( )
{
	framework_dir="$output_directory/$1.framework"
	mkdir -p "$framework_dir" &&
	mkdir -p "$framework_dir/Versions/$framework_version/Headers" &&
	mkdir -p "$framework_dir/Versions/$framework_version/PlugIns" &&
	mkdir -p "$framework_dir/Versions/$framework_version/Resources/English.lproj/Documentation" &&
	mkdir -p "$framework_dir/Versions/$framework_version/Resources/Portuguese.lproj/Documentation" &&
	mkdir -p "$framework_dir/Versions/$framework_version/Libraries" &&
	ln -sf "$framework_version" "$framework_dir/Versions/Current" &&
	ln -sf "Versions/Current/Headers" "$framework_dir/Headers" &&
	ln -sf "Versions/Current/PlugIns" "$framework_dir/PlugIns" &&
	ln -sf "Versions/Current/Resources" "$framework_dir/Resources" &&
	ln -sf "Versions/Current/Libraries" "$framework_dir/Libraries"
}

make_framework_headers( )
{
	tl=`cat -n $1/Makefile.am | cut -f 1 | tail -n 1` &&
	hl=`cat -n $1/Makefile.am | grep HEADERS | cut -f 1` &&
	headers=`cat $1/Makefile.am | tail -n $((tl-hl)) | cut -f 2` &&
	for i in $headers
	do	basedir=`dirname $1/$i | sed -e "s|$1||g"` &&
		mkdir -p $2.framework/Versions/Current/Headers/$basedir &&
		echo $1 > header_temp.$$
		header_path_first=`cut -d '/' -f 5 header_temp.$$`
		header_path_second=`cut -d '/' -f 6 header_temp.$$`
		header_path="$header_path_first/$header_path_second"
	
		if [ -r $1/$i ]
		then 
			cp $1/$i current_header.$$
	
			deplist="openpluginlib/pl openimagelib/il openmedialib/ml openassetlib/al openobjectlib/sg"
			for j in $deplist
			do	case $j in
				openpluginlib* ) 
					sed -e "s|$j|OpenPluginLib|g" -i '' current_header.$$ > $2.framework/Versions/Current/Headers/$i
					;;
				openimagelib* ) 
					sed -e "s|$j|OpenImageLib|g" -i '' current_header.$$ > $2.framework/Versions/Current/Headers/$i
					;;
				openmedialib* ) 
					sed -e "s|$j|OpenMediaLib|g" -i '' current_header.$$ > $2.framework/Versions/Current/Headers/$i
					;;
				openobjectlib* ) 
					sed -e "s|$j|OpenObjectLib|g" -i '' current_header.$$ > $2.framework/Versions/Current/Headers/$i
					;;
				openassetlib* ) 
					sed -e "s|$j|OpenAssetLib|g" -i '' current_header.$$ > $2.framework/Versions/Current/Headers/$i
					;;
				esac
			done

			sed -e "s|$header_path|$2|g" current_header.$$ > $2.framework/Versions/Current/Headers/$i
		
			rm current_header.$$
		fi
		
		rm header_temp.$$
	done	
}

make_framework_libs( )
{
	libs=`ls $2/.libs/*.?.dylib`
	for i in $libs
	do	if [ ! -L $i ]
		then	ln -sf Versions/Current/$1 $1.framework/$1 &&
				cp $i $1.framework/Versions/Current/$1 &&
				install_name_tool -id $install_name_prefix/$1.framework/Versions/$framework_version/$1 $1.framework/$1
		fi
	done	
}

make_framework_python_libs( )
{
	if [ -d $2 ]
	then
		libs=`ls $2/.libs/*.?.dylib`
		for i in $libs
		do	if [ ! -L $i ]
			then	cp $i $1.framework/Versions/$framework_version/PlugIns/$1.so &&
					install_name_tool -id $install_name_prefix/$1.framework/Versions/$framework_version/$1 $1.framework/$1
			fi
		done
	fi
}

make_framework_deps( )
{
	deplist="GLEW boost"
	for i in $deplist
	do	depname=`otool -L $1.framework/$1 | grep $i | cut -d ' ' -f 1`
		for j in $depname
		do	depdir=`dirname $j`
			libname="$j"
			if [ $depdir == "." ]
			then	libname="/usr/local/lib/$depdir/$j"
			fi
			base=`basename $j`
			cp $libname $1.framework/Versions/$framework_version/Libraries &&
			install_name_tool -id $install_name_prefix/$1.framework/Versions/$framework_version/$base $1.framework/Versions/$framework_version/Libraries/$base &&
			install_name_tool -change $j $install_name_prefix/$1.framework/Versions/$framework_version/Libraries/$base $1.framework/$1
		done
	done
}

make_framework_resources( )
{
	cp ./Info.plist.template $1.framework/Resources/Info.plist &&
	cp ./InfoPlist.strings.template $1.framework/Resources/English.lproj/InfoPlist.strings

	if [ -d "$2/../plugins" ]
	then	plugins=`ls $2/../plugins`
			for i in $plugins
			do	if [ -d $2/../plugins/$i/.libs ]
				then cp $2/../plugins/$i/*.opl $1.framework/PlugIns
				fi
			done
	fi
}

make_framework_libs_deps( )
{
	deplist="/usr/local/lib"
	for i in $deplist
	do	libs=`ls $1.framework/Libraries`
		for j in $libs
		do	if [ ! -L $1.framework/Versions/$framework_version/Libraries/$j ]
			then	depname=`otool -L $1.framework/Libraries/$j | grep $i | cut -d ' ' -f 1`
					for k in $depname
					do	base=`basename $k`
						if [ ! -e $1.framework/Versions/$framework_version/Libraries/$base ]
						then	cp $k $1.framework/Versions/$framework_version/Libraries
								install_name_tool -id $install_name_prefix/$1.framework/Versions/$framework_version/$base \
									$1.framework/Versions/$framework_version/Libraries/$base
						fi
						install_name_tool -change $k \
							$install_name_prefix/$1.framework/Versions/$framework_version/Libraries/$base \
							$1.framework/Versions/$framework_version/Libraries/$j
					done
			fi
		done
	done
}

make_framework_plugins_deps( )
{
	deplist="GLEW boost /usr/local/lib"
	for i in $deplist
	do	plugins=`ls $1.framework/PlugIns`
		for j in $plugins
		do	if [ ! -L $1.framework/PlugIns/$j ]
			then	install_name_tool -id $install_name_prefix/$1.framework/Versions/$framework_version/$j \
						$1.framework/Versions/$framework_version/PlugIns/$j
					depname=`otool -L $1.framework/PlugIns/$j | grep $i | cut -d ' ' -f 1`
					for k in $depname
					do	depdir=`dirname $k`
						libname="$k"
						if [ $depdir == "." ]
						then	libname="/usr/local/lib/$depdir/$k"
						fi
						base=`basename $k`
						if [ ! -e $1.framework/Versions/$framework_version/Libraries/$base ]
						then
							if [ -e $libname ]
							then	cp $libname $1.framework/Versions/$framework_version/Libraries
									install_name_tool -id $install_name_prefix/$1.framework/Versions/$framework_version/$base \
										$1.framework/Versions/$framework_version/Libraries/$base
									install_name_tool -change $k \
										$install_name_prefix/$1.framework/Versions/$framework_version/Libraries/$base $1.framework/$1
							fi
						fi
						install_name_tool -change $k \
							$install_name_prefix/$1.framework/Versions/$framework_version/Libraries/$base \
							$1.framework/Versions/$framework_version/PlugIns/$j
					done
			fi
		done
	done
}

make_framework_plugins( )
{
	if [ -d "$1/../plugins" ]
	then	plugins=`ls $1/../plugins`
			for i in $plugins
			do	if [ -d $1/../plugins/$i/.libs ]
				then	cp -R $1/../plugins/$i/.libs/*.dylib $2.framework/PlugIns
				fi
			done
	fi
}

make_framework_interdeps( )
{
	deplist="openpluginlib_pl openimagelib_il openmedialib_ml openassetlib_al openobjectlib_sg"
	for i in $deplist
	do	plugins=`ls $1.framework/PlugIns/*.* 2> /dev/null`
		for j in $plugins
		do	depname=`otool -L $j | grep $i | cut -d ' ' -f 1`
			if [ ! -z $depname ]
			then	case $i in
						openpluginlib_pl )
							install_name_tool -change $depname \
								$install_name_prefix/OpenPluginLib.framework/Versions/$framework_version/OpenPluginLib $j
						;;
						openimagelib_il )
							install_name_tool -change $depname \
								$install_name_prefix/OpenImageLib.framework/Versions/$framework_version/OpenImageLib $j
						;;
						openmedialib_ml )
							install_name_tool -change $depname \
								$install_name_prefix/OpenMediaLib.framework/Versions/$framework_version/OpenMediaLib $j
						;;
						openobjectlib_sg )
							install_name_tool -change $depname \
								$install_name_prefix/OpenObjectLib.framework/Versions/$framework_version/OpenObjectLib $j
						;;
						openassetlib_al )
							install_name_tool -change $depname \
								$install_name_prefix/OpenAssetLib.framework/Versions/$framework_version/OpenAssetLib $j
						;;
					esac
			fi
		done
	done	
}

make_framework( )
{
    rm -rf $1.framework
	make_framework_dir $1 &&
	make_framework_headers $2 $1 &&
	make_framework_libs $1 $2 &&
	make_framework_python_libs $1 "$2/../py" &&
	make_framework_deps $1 &&
	make_framework_plugins $2 $1 &&
	make_framework_interdeps $1 &&
	make_framework_plugins_deps $1 &&
	make_framework_libs_deps $1 &&
	make_framework_resources $1 $2
}

make_umbrella_framework_deps( )
{
	deplist="openpluginlib_pl openimagelib_il openmedialib_ml openassetlib_al openobjectlib_sg"
	for i in $deplist
	do	depname=`otool -L $1 | grep $i | cut -d ' ' -f 1`
		if [ ! -z $depname ]
		then	case $i in
					openpluginlib_pl )
						install_name_tool -change $depname \
							"$install_name_prefix"/OpenLibraries.framework/Versions/$framework_version/Frameworks/OpenPluginLib.framework/Versions/$framework_version/OpenPluginLib \
							$1							
					;;
					openimagelib_il )
						install_name_tool -change $depname \
							"$install_name_prefix"/OpenLibraries.framework/Versions/$framework_version/Frameworks/OpenImageLib.framework/Versions/$framework_version/OpenImageLib \
							$1							
					;;
					openmedialib_ml )
						install_name_tool -change $depname \
							"$install_name_prefix"/OpenLibraries.framework/Versions/$framework_version/Frameworks/OpenMediaLib.framework/Versions/$framework_version/OpenMediaLib \
							$1					
					;;
					openobjectlib_sg )
						install_name_tool -change $depname \
							"$install_name_prefix"/OpenLibraries.framework/Versions/$framework_version/Frameworks/OpenObjectLib.framework/Versions/$framework_version/OpenObjectLib \
							$1					
					;;
					openassetlib_al )
						install_name_tool -change $depname \
							"$install_name_prefix"/OpenLibraries.framework/Versions/$framework_version/Frameworks/OpenAssetLib.framework/Versions/$framework_version/OpenAssetLib \
							$1					
					;;
				esac
		fi
	done
}

make_umbrella_framework_python_deps( )
{
	install_name_tool -id $install_name_prefix/OpenLibraries.framework/Frameworks/$1.framework/Versions/A/PlugIns/$1.so $2
	fmwklist="OpenPluginLib OpenImageLib OpenMediaLib OpenAssetLib OpenObjectLib"
	for i in $fmwklist
	do	install_name_tool -change $install_name_prefix/$i.framework/Versions/A/$i \
			$install_name_prefix/OpenLibraries.framework/Frameworks/$i.framework/Versions/A/$i $2
	done
}

update_umbrella_framework_install_names( )
{
	install_name_tool -id $install_name_prefix/OpenLibraries.framework/Versions/A/Frameworks/$1.framework/Versions/A/$1 \
			OpenLibraries.framework/Versions/A/Frameworks/$1.framework/$1
			
	libs=`ls OpenLibraries.framework/Versions/A/Frameworks/$1.framework/Versions/A/Libraries/*.dylib`
	for i in $libs
	do	install_name_tool -id $install_name_prefix/OpenLibraries.framework/Versions/A/Frameworks/$1.framework/Versions/A/Libraries/`basename $i` $i
	done
	
	if [ $1 != "OpenPluginLib" ]
	then libs=`ls OpenLibraries.framework/Versions/A/Frameworks/$1.framework/Versions/A/PlugIns/*.dylib`
		 for i in $libs
		 do	install_name_tool -id $install_name_prefix/OpenLibraries.framework/Versions/A/Frameworks/$1.framework/Versions/A/PlugIns/`basename $i` $i
		 done
		 
		 depnames="$install_name_prefix/OpenPluginLib.framework $install_name_prefix/OpenImageLib.framework $install_name_prefix/OpenMediaLib.framework $install_name_prefix/OpenAssetLib.framework $install_name_prefix/OpenObjectLib.framework"
		 for i in $libs
		 do	for j in $depnames
			do	deps=`otool -L $i | grep $j | cut -d ' ' -f 1`
				for k in $deps
				do	if [ `basename $k` != "OpenPluginLib" ] && [ `basename $k` != "OpenImageLib" ] && [ `basename $k` != "OpenMediaLib" ] && [ `basename $k` != "OpenAssetLib" ] && [ `basename $k` != "OpenObjectLib" ]
					then install_name_tool -change $k \
							$install_name_prefix/OpenLibraries.framework/Versions/A/Frameworks/$1.framework/Versions/A/Libraries/`basename $k` $i
					else install_name_tool -change $k \
							$install_name_prefix/OpenLibraries.framework/Versions/A/Frameworks/`basename $k`.framework/Versions/A/`basename $k` $i
					fi
				done
			done
		 done
	fi
	
	libs=`ls OpenLibraries.framework/Versions/A/Frameworks/$1.framework/Versions/A/Libraries/*.dylib`
	for i in $libs
	do	deps=`otool -L $i | grep $install_name_prefix/$1.framework | cut -d ' ' -f 1`
		for k in $deps
		do	install_name_tool -change $k \
					$install_name_prefix/OpenLibraries.framework/Versions/A/Frameworks/$1.framework/Versions/A/Libraries/`basename $k` $i
		done
	done
}

make_umbrella_framework( )
{
	rm -rf "./OpenLibraries.framework"
	make_framework "OpenLibraries" "./src/umbrella_framework"
	
	framework_dir="$output_directory/OpenLibraries.framework"
	mkdir -p "$framework_dir/Versions/$framework_version/Frameworks"
	ln -sf "Versions/Current/Frameworks" "$framework_dir/Frameworks"
	
	cp -R "./OpenPluginLib.framework" "$framework_dir/Versions/$framework_version/Frameworks" 
	cp -R "./OpenImageLib.framework" "$framework_dir/Versions/$framework_version/Frameworks"
	cp -R "./OpenMediaLib.framework" "$framework_dir/Versions/$framework_version/Frameworks"
	cp -R "./OpenAssetLib.framework" "$framework_dir/Versions/$framework_version/Frameworks"
	cp -R "./OpenObjectLib.framework" "$framework_dir/Versions/$framework_version/Frameworks"
	
	make_umbrella_framework_deps "OpenLibraries.framework/OpenLibraries" &&
	make_umbrella_framework_deps "OpenLibraries.framework/Frameworks/OpenPluginLib.framework/PlugIns/OpenPluginLib.so" &&
	make_umbrella_framework_deps "OpenLibraries.framework/Frameworks/OpenImageLib.framework/PlugIns/OpenImageLib.so" &&
	make_umbrella_framework_deps "OpenLibraries.framework/Frameworks/OpenMediaLib.framework/PlugIns/OpenMediaLib.so" &&
	make_umbrella_framework_deps "OpenLibraries.framework/Frameworks/OpenAssetLib.framework/PlugIns/OpenAssetLib.so" &&
	make_umbrella_framework_deps "OpenLibraries.framework/Frameworks/OpenObjectLib.framework/PlugIns/OpenObjectLib.so"

	deplist="GLEW boost"
	for i in $deplist
	do	fmwklist="OpenPluginLib OpenImageLib OpenMediaLib OpenAssetLib OpenObjectLib"
	    for j in $fmwklist
		do	depname=`otool -L OpenLibraries.framework/Frameworks/$j.framework/$j | grep $i | cut -d ' ' -f 1`
			for k in $depname
			do  base=`basename $k`
				install_name_tool -change $k \
					$install_name_prefix/OpenLibraries.framework/Versions/$framework_version/Libraries/$base \
					OpenLibraries.framework/Versions/$framework_version/Frameworks/$j.framework/$j				
			done			
		done
			
		for j in $fmwklist
		do	depname=`otool -L OpenLibraries.framework/Frameworks/$j.framework/PlugIns/$j.so | grep $i | cut -d ' ' -f 1`
			for k in $depname
			do	base=`basename $k`
				install_name_tool -change $k \
					$install_name_prefix/OpenLibraries.framework/Versions/$framework_version/Libraries/$base \
					OpenLibraries.framework/Frameworks/$j.framework/PlugIns/$j.so
			done
		done
		
		for j in $fmwklist
		do	if [ $j == "OpenPluginLib" ] 
			then continue
			fi
			libraries=`ls OpenLibraries.framework/Frameworks/$j.framework/PlugIns/*.dylib`
			for lib in $libraries
			do	depname=`otool -L $lib | grep $i | cut -d ' ' -f 1`
				for k in $depname
				do	base=`basename $k`
					install_name_tool -change $k \
						$install_name_prefix/OpenLibraries.framework/Versions/$framework_version/Libraries/$base $lib
				done
			done
		done
	done
	
	update_umbrella_framework_install_names "OpenPluginLib"
	update_umbrella_framework_install_names "OpenImageLib"
	update_umbrella_framework_install_names "OpenMediaLib"
	update_umbrella_framework_install_names "OpenAssetLib"
	update_umbrella_framework_install_names "OpenObjectLib"
		
	make_umbrella_framework_python_deps "OpenPluginLib" "OpenLibraries.framework/Frameworks/OpenPluginLib.framework/PlugIns/OpenPluginLib.so"
	make_umbrella_framework_python_deps "OpenImageLib" "OpenLibraries.framework/Frameworks/OpenImageLib.framework/PlugIns/OpenImageLib.so"
	make_umbrella_framework_python_deps "OpenMediaLib" "OpenLibraries.framework/Frameworks/OpenMediaLib.framework/PlugIns/OpenMediaLib.so"
	make_umbrella_framework_python_deps "OpenAssetLib" "OpenLibraries.framework/Frameworks/OpenAssetLib.framework/PlugIns/OpenAssetLib.so"
	
	# Umbrella Framework Header
	deplist="openpluginlib/pl openimagelib/il openmedialib/ml openassetlib/al"
	for i in $deplist
	do	case $i in
			openpluginlib* ) 
				sed -e "s|$i|OpenPluginLib|g" -i '' OpenLibraries.framework/Versions/$framework_version/Headers/openlibraries.hpp
				;;
			openimagelib* ) 
				sed -e "s|$i|OpenImageLib|g" -i '' OpenLibraries.framework/Versions/$framework_version/Headers/openlibraries.hpp
				;;
			openmedialib* ) 
				sed -e "s|$i|OpenMediaLib|g" -i '' OpenLibraries.framework/Versions/$framework_version/Headers/openlibraries.hpp
				;;
			openassetlib* ) 
				sed -e "s|$i|OpenAssetLib|g" -i '' OpenLibraries.framework/Versions/$framework_version/Headers/openlibraries.hpp
				;;
		esac
	done
	
	check_status $?
}

make_installer( )
{
    /Developer/Applications/Utilities/PackageMaker.app/Contents/MacOS/PackageMaker -build -proj $1.pmproj -p $1.pkg &&
	hdiutil create -ov -srcfolder $1.pkg $1.dmg -fs HFS+ -volname "$1"
}

strip_framework_symbols( )
{
	if [ "$strip_symbols" -eq 1 ]
	then
		echo -n "        Stripping symbols..."
		strip -x OpenLibraries.framework/OpenLibraries &&
		strip -x OpenLibraries.framework/Libraries/* &&
		strip -x OpenLibraries.framework/Frameworks/OpenPluginLib.framework/OpenPluginLib &&
		strip -x OpenLibraries.framework/Frameworks/OpenPluginLib.framework/Libraries/*.dylib &&
		strip -x OpenLibraries.framework/Frameworks/OpenPluginLib.framework/PlugIns/*.so &&
		strip -x OpenLibraries.framework/Frameworks/OpenImageLib.framework/OpenImageLib &&
		strip -x OpenLibraries.framework/Frameworks/OpenImageLib.framework/Libraries/*.dylib &&
		strip -x OpenLibraries.framework/Frameworks/OpenImageLib.framework/PlugIns/*.so &&
		strip -x OpenLibraries.framework/Frameworks/OpenImageLib.framework/PlugIns/*.dylib &&
		strip -x OpenLibraries.framework/Frameworks/OpenMediaLib.framework/OpenMediaLib &&
		strip -x OpenLibraries.framework/Frameworks/OpenMediaLib.framework/Libraries/*.dylib &&
		strip -x OpenLibraries.framework/Frameworks/OpenMediaLib.framework/PlugIns/*.so &&
		strip -x OpenLibraries.framework/Frameworks/OpenMediaLib.framework/PlugIns/*.dylib &&
		strip -x OpenLibraries.framework/Frameworks/OpenAssetLib.framework/OpenAssetLib &&
		strip -x OpenLibraries.framework/Frameworks/OpenAssetLib.framework/Libraries/*.dylib &&
		strip -x OpenLibraries.framework/Frameworks/OpenAssetLib.framework/PlugIns/*.so &&
		strip -x OpenLibraries.framework/Frameworks/OpenAssetLib.framework/PlugIns/*.dylib &&
		strip -x OpenLibraries.framework/Frameworks/OpenObjectLib.framework/OpenObjectLib &&
		strip -x OpenLibraries.framework/Frameworks/OpenObjectLib.framework/Libraries/*.dylib &&
		strip -x OpenLibraries.framework/Frameworks/OpenObjectLib.framework/PlugIns/*.dylib

		check_status $?
	fi
}

make_framework_libraries_unique( )
{
	libraries=`ls OpenLibraries.framework/Versions/$framework_version/Libraries/*.dylib`
	for i in $libraries
	do	install_name_tool -id $install_name_prefix/OpenLibraries.framework/Versions/$framework_version/Libraries/`basename $i` $i
		if [ -e OpenLibraries.framework/Versions/$framework_version/Frameworks/$1.framework/Versions/$framework_version/Libraries/`basename $i` ]
		then rm -rf OpenLibraries.framework/Versions/$framework_version/Frameworks/$1.framework/Versions/$framework_version/Libraries/`basename $i`
		fi
	done
}

if [ "$build_frameworks" -eq 1 ] && [ "$PLATFORM" = "Darwin" ] 
then
	echo "Building OS/X Frameworks..."
	
	echo "    OpenPluginLib..." &&
	make_framework "OpenPluginLib" "./src/openpluginlib/pl" &&
	echo "    OpenImageLib..." &&
	make_framework "OpenImageLib" "./src/openimagelib/il" &&
	echo "    OpenMediaLib..." &&
	make_framework "OpenMediaLib" "./src/openmedialib/ml" &&
	echo "    OpenAssetLib..." &&
	make_framework "OpenAssetLib" "./src/openassetlib/al" &&
	echo -n "    OpenObjectLib..." &&
	make_framework "OpenObjectLib" "./src/openobjectlib/sg"
	check_status $?
	
	if [ "$build_umbrella" -eq 1 ]
	then  echo "    Building Umbrella Framework..."
		  echo -n "        OpenLibraries..."
		  make_umbrella_framework &&
		  strip_framework_symbols &&
		  make_framework_libraries_unique "OpenPluginLib" &&
		  make_framework_libraries_unique "OpenImageLib" &&
		  make_framework_libraries_unique "OpenMediaLib" &&
		  make_framework_libraries_unique "OpenAssetLib" &&
		  make_framework_libraries_unique "OpenObjectLib"
		  check_status $?
		  
		  if [ x"$USER" = x"root" ]
		  then chown -Rh root:admin OpenLibraries.framework
		  fi
	fi
	
	if [ "$build_installer" -eq 1 ]
	then	echo "    Building packages..."			
			make_installer OpenLibraries.framework
			check_status $?
	fi
fi

exit 0
